# widgets.py
#
# Copyright 2020 Morgan McMillian
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import gi
import os
import pnutpy
import logging

gi.require_version('Gdk', '3.0')
gi.require_version('Gtk', '3.0')
from gi.repository import GObject, Gdk, Gtk

gi.require_version('Handy', '1')
from gi.repository import Handy

class ComposeWindow(Handy.Window):

    def __init__(self):
        super().__init__(modal=True)
        self.set_default_size(500, 300)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)

        box = Gtk.Box(orientation='vertical')
        header = Handy.HeaderBar(show_close_button=False)

        cancel_button = Gtk.Button(label='Cancel')
        cancel_button.connect('clicked', self.cancel_post)
        post_button = Gtk.Button(label='Post')
        post_button.connect('clicked', self.send_post)

        header.set_title('New Post')
        header.pack_start(cancel_button)
        header.pack_end(post_button)

        scroller = Gtk.ScrolledWindow(halign='fill')
        textarea = Gtk.TextView(
            left_margin=8,
            right_margin=8,
            top_margin=8,
            bottom_margin=8
        )
        textarea.set_wrap_mode(Gtk.WrapMode.WORD)
        scroller.add(textarea)
        self.buffer = textarea.get_buffer()

        self.max_length = 256
        actionbar = Gtk.ActionBar()
        self.counter_label = Gtk.Label()
        self.counter_label.set_text(str(self.max_length))
        actionbar.pack_end(self.counter_label)

        self.buffer.connect('changed', self.validate)

        box.pack_start(header, False, False, 0)
        box.pack_start(scroller, True, True, 0)
        box.pack_end(actionbar, False, False, 0)
        self.add(box)

        self.show_all()

        self.connect('key-release-event', self.on_key_release)

    def on_key_release(self, widget, ev, data=None):
        if ev.keyval == Gdk.KEY_Escape:
            self.close()

    def validate(self, widget):
        self.counter = self.max_length - self.buffer.get_char_count()
        self.counter_label.set_text(str(self.counter))

    def send_post(self, button):
        start = self.buffer.get_start_iter()
        end = self.buffer.get_end_iter()
        text = self.buffer.get_text(start, end, False)
        pnutpy.api.create_post(data={'text': text})
        self.close()

    def cancel_post(self, button):
        self.close()

class LoginPage(Gtk.Box):

    __gsignals__ = {
        'login': (GObject.SIGNAL_RUN_FIRST, None, (str,))
    }

    def __init__(self):
        super().__init__(orientation='vertical')
        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)

        client_id = "1PiUzxfX_CQxKvtz93lUzPX9-FMtz-va"
        redirect_uri = "urn:ietf:wg:oauth:2.0:oob"
        scope =  "basic,stream,write_post,follow,presence,messages,files,polls"

        uri = "https://pnut.io/oauth/authenticate"
        uri += "?client_id=" + client_id
        uri += "&redirect_uri=" + redirect_uri
        uri += "&scope=" + scope
        uri += "&response_type=token"

        self.login_button = Gtk.LinkButton.new_with_label(uri, "Log In to pnut.io")
        self.login_button.connect("clicked", self.prompt_code)

        self.set_center_widget(self.login_button)

    def prompt_code(self, button):
        self.remove(self.login_button)

        label = Gtk.Label()
        label.set_markup('<span font="20">Enter authorization code</span>')
        self.code = Gtk.Entry()

        paste_button = Gtk.Button(label="Paste from clipboard")
        paste_button.connect("clicked", self.paste_code)

        cancel_button = Gtk.Button(label="Cancel")
        cancel_button.connect("clicked", self.cancel_login)
        confirm_button = Gtk.Button(label="Confirm")
        confirm_button.connect("clicked", self.confirm_login)
        lbox = Gtk.Box(orientation='horizontal')
        lbox.pack_start(cancel_button, True, True, 0)
        lbox.pack_start(confirm_button, True, True, 0)

        vbox = Gtk.Box(orientation='vertical')
        vbox.pack_start(label, False, False, 10)
        vbox.pack_start(self.code, False, False, 10)
        vbox.pack_start(paste_button, False, False, 10)
        vbox.add(lbox)

        hbox = Gtk.Box(orientation='horizontal')
        hbox.set_center_widget(vbox)

        self.set_center_widget(hbox)
        self.show_all()

    def paste_code(self, button):
        text = self.clipboard.wait_for_text()
        if text is not None:
            self.code.set_text(text)

    def cancel_login(self, button):
        # TODO: something actually useful here
        logging.debug("uh cancel i guess")

    def confirm_login(self, button):
        code = self.code.get_text()
        self.emit('login', code)

class Timeline(Gtk.Box):

    __gsignals__ = {
        'refresh': (GObject.SIGNAL_RUN_FIRST, None, ())
    }

    def __init__(self, stream):
        super().__init__(orientation='vertical')

        scroller = Gtk.ScrolledWindow(
            halign='fill',
            kinetic_scrolling=True
        )
        self.view = Gtk.ListBox(
            selection_mode=Gtk.SelectionMode.NONE
        )
        scroller.add(self.view)
        self.pack_start(scroller, True, True, 0)

        self.stream = stream
        self.load_timeline()

    def load_timeline(self):
        if self.stream == 'unified':
            posts, meta = pnutpy.api.users_post_streams_unified()
        elif self.stream == 'mentions':
            posts, meta = pnutpy.api.users_mentioned_posts('me')
        elif self.stream == 'bookmarks':
            posts, meta = pnutpy.api.users_bookmarked_posts('me')
        else:
            posts, meta = pnutpy.api.posts_streams_global()

        for item in posts:
            if 'is_deleted' in item:
                continue
            self.view.add(PostItem(item))

    def do_refresh(self):
        rows = self.view.get_children()
        for item in rows:
            self.view.remove(item)
        self.load_timeline()
        self.show_all()

class PostItem(Gtk.ListBoxRow):

    def __init__(self, post):
        super(Gtk.ListBoxRow, self).__init__()
        self.post = post

        self.box = Gtk.Box(orientation='vertical')
        self.add(self.box)

        # name container
        self.name_box = Gtk.Box(orientation='vertical')
        self.username = Gtk.Label(label="@" + post.user.username, xalign=0)
        self.name = Gtk.Label(xalign=0)
        if 'name' in post.user:
            self.name.set_markup(f"<b>{post.user.name}</b>")
        self.name_box.pack_start(self.name, True, True, 0)
        self.name_box.pack_start(self.username, True, True, 0)

        # header container
        self.h_box = Gtk.Box(orientation='horizontal')
        self.avatar = Handy.Avatar(size=32)
        # TODO: get the actual image
        self.h_box.pack_start(self.avatar, False, False, 18)
        self.h_box.pack_start(self.name_box, False, False, 0)

        # content container
        self.c_box = Gtk.Box(orientation='horizontal')
        self.content = Gtk.Label(wrap=True, xalign=0)
        # TODO: parse content links
        if 'content' in post:
            self.content.set_text(post.content.text)
        # TODO: add media
        self.c_box.pack_start(self.content, True, True, 18)

        self.box.pack_start(self.h_box, True, True, 10)
        self.box.pack_start(self.c_box, True, True, 10)

